@@ -7,7 +7,7 @@
  *  You may distribute this under the terms of either the GNU General Public
  *  License or the Artistic License, as specified in the Perl README file.
  *
- *  $Id: dbdimp.c,v 1.27 2005/04/02 06:50:05 rlippan Exp $
+ *  $Id: dbdimp.c,v 1.29 2005/04/22 22:09:55 capttofu Exp $
  */
 
 
@@ -1630,7 +1630,7 @@ my_ulonglong mysql_st_internal_execute(SV* h, SV* statement, SV* attribs,
     D_imp_sth(h);
     D_imp_dbh_from_sth;
     STRLEN slen;
-    my_ulonglong rows;
+    my_ulonglong rows=0;
 
     char* sbuf = SvPV(statement, slen);
 
@@ -1716,16 +1716,9 @@ my_ulonglong mysql_st_internal_execute(SV* h, SV* statement, SV* attribs,
     else
       rows= mysql_num_rows(*cdaPtr);
 
-    if ((long long)rows == -1)
-    {      
-      if (dbis->debug >= 2) 
-	    PerlIO_printf(DBILOGFP,
-                         "      <- mysql_st_internal_execute ERROR: returning -1\n");
-         return(-1);
-    }
     if (dbis->debug >= 2) 
       PerlIO_printf(DBILOGFP,
-              "      <- mysql_st_internal_execute ERROR: returning -1\n");
+                    "      <- mysql_st_internal_execute returning rows %llu\n", rows);
     return(rows);
 }
 
@@ -1749,6 +1742,8 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
     D_imp_dbh_from_sth;
     SV** statement;
     int i;
+
+    char actual_row_num[64];
 #if defined (dTHR)
     dTHR;
 #endif
@@ -1773,17 +1768,22 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
     }
 
     statement = hv_fetch((HV*) SvRV(sth), "Statement", 9, FALSE);
-    if ((imp_sth->row_num =
-	     mysql_st_internal_execute(sth, *statement, NULL,
-				       DBIc_NUM_PARAMS(imp_sth),
-				       imp_sth->params,
-				       &imp_sth->cda,
- 				       &imp_dbh->mysql,
-				       imp_sth->use_mysql_use_result))
-	!= -2) {
-	if (!imp_sth->cda) {
+    imp_sth->row_num= mysql_st_internal_execute(
+                                                sth, *statement, NULL,
+                                                DBIc_NUM_PARAMS(imp_sth),
+                                                imp_sth->params,
+                                                &imp_sth->cda,
+                                                &imp_dbh->mysql,
+                                                imp_sth->use_mysql_use_result);
+
+    if (imp_sth->row_num+1 != (my_ulonglong)-1)
+    {
+	if (!imp_sth->cda)
+        {
 	    imp_sth->insertid = mysql_insert_id(&imp_dbh->mysql);
-	} else {
+	}
+        else
+        {
 	    /** Store the result in the current statement handle */
 	    DBIc_ACTIVE_on(imp_sth);
 	    DBIc_NUM_FIELDS(imp_sth) = mysql_num_fields(imp_sth->cda);
@@ -1791,15 +1791,19 @@ int dbd_st_execute(SV* sth, imp_sth_t* imp_sth) {
 	}
     }
 
-    if (dbis->debug >= 2) {
-        PerlIO_printf(DBILOGFP, "    <- dbd_st_execute %d rows\n",
-		      imp_sth->row_num);
+    if (dbis->debug >= 2)
+    {
+      /* 
+        PerlIO_printf doesn't always handle imp_sth->row_num %llu 
+        consistantly!!
+      */
+      sprintf(actual_row_num, "%llu", imp_sth->row_num);
+      PerlIO_printf(DBILOGFP,
+                    "    <- dbd_st_execute returning imp_sth->row_num %s\n",
+                    actual_row_num);
     }
 
-    if ((long long) imp_sth->row_num == -1)
-      return -1;
-    else
-      return (int) imp_sth->row_num;
+    return (int) imp_sth->row_num;
 }
 
 
@@ -1960,10 +1964,6 @@ void dbd_st_destroy(SV* sth, imp_sth_t* imp_sth) {
      */
     for (i = 0;  i < AV_ATTRIB_LAST;  i++) {
 	if (imp_sth->av_attr[i]) {
-#ifdef DEBUGGING_MEMORY_LEAK
-	    PerlIO_printf("DESTROY: Decrementing refcnt: old = %d\n",
-			  SvREFCNT(imp_sth->av_attr[i]));
-#endif
 	    SvREFCNT_dec(imp_sth->av_attr[i]);
 	}
 	imp_sth->av_attr[i] = Nullav;
@@ -2327,6 +2327,7 @@ int dbd_st_blob_read (SV *sth, imp_sth_t *imp_sth, int field, long offset,
 int dbd_bind_ph (SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 		 IV sql_type, SV *attribs, int is_inout, IV maxlen) {
     int paramNum = SvIV(param);
+    char err_msg[128];
 
     if (paramNum <= 0  ||  paramNum > DBIc_NUM_PARAMS(imp_sth)) {
         do_error(sth, JW_ERR_ILLEGAL_PARAM_NUM,
@@ -2334,6 +2335,28 @@ int dbd_bind_ph (SV *sth, imp_sth_t *imp_sth, SV *param, SV *value,
 	return FALSE;
     }
 
+    /* 
+      This fixes the bug whereby no warning was issued upone binding a 
+      non-numeric as numeric
+    */
+    if (sql_type == SQL_NUMERIC  ||
+        sql_type == SQL_DECIMAL  ||
+        sql_type == SQL_INTEGER  ||
+        sql_type == SQL_SMALLINT ||
+        sql_type == SQL_FLOAT    ||
+        sql_type == SQL_REAL     ||
+        sql_type == SQL_DOUBLE)
+    {
+      if (! looks_like_number(value))
+      {
+        sprintf(err_msg,
+                "Binding non-numeric field %d, value %s as a numeric!",
+                paramNum, neatsvpv(value,0));
+        do_error(sth, JW_ERR_ILLEGAL_PARAM_NUM, err_msg);
+      }
+    }
+
+
     if (is_inout) {
         do_error(sth, JW_ERR_NOT_IMPLEMENTED,
 		       "Output parameters not implemented");
